<?php
require_once('mypdo.include.php') ;
require_once('picto.class.php') ;

/**
 * Classe d'Exception concernant les connexions de la Classe Utilisateur
 */
class AuthenticationException extends Exception { }

/**
 * Classe d'Exception concernant les rcupration de la Classe Utilisateur dans les donnes de session
 */
class NotInSessionException extends Exception { }

/**
 * Classe d'Exception concernant le dmarrage d'une session
 */
class SessionException extends Exception { }

/**
 * Utilisateur permettant d'effectuer des connexions client/serveur
 * Utilisateur issu de la base de donnes (table tb_user)
 */
class User {
    /**
     * Identifiant base de donnes
     * @var string $id
     */
    protected $id        = null ;

    /**
     * Nom
     * @var string $lastName
     */
    protected $lastName  = null ;

    /**
     * Prnom
     * @var string $firstName
     */
    protected $firstName = null ;

    /**
     * Login
     * @var string $login
     */
    protected $login     = null ;

    /**
     * Numro de tlphone
     * @var string $phone
     */
    protected $phone     = null ;

    /**
     * Savoir si le user est un admin ou non
     * @var int $admin
     */
    protected $admin     = null ;

    /**
     * Cl de session  partir de laquelle les donnes sont stockes
     */
    const session_key = "__user__" ;

    /**
     * Constructeur priv
     */
    private function __construct() {
    }

    /**
     * Affiche le prnom de l'Utilisateur (donne le contenu de l'attribut d'instance)
     *
     * @return string prnom
     */
    public function firstName() {
        return $this->firstName ;
    }

    /**
     * Affichage
     *
     * @return string code HTML prsentant les attributs de l'utilisateur
     */
    public function profile() {
            // Affichage
            return <<<HTML
    <div><span>id       </span> : <span>{$this->id} </span></div>
    <div><span>Nom      </span> : <span>{$this->lastName} </span></div>
    <div><span>Prenom   </span> : <span>{$this->firstName}</span></div>
    <div><span>Login    </span> : <span>{$this->login}    </span></div>
    <div><span>Admin</span> : <span>{$this->admin}    </span></div>
HTML;
    }

    /**
     * Production d'un formulaire de connexion
     * @param string $action URL cible du formulaire
     * @param string $submitText texte du bouton d'envoi
     *
     * @return string code HTML du formulaire
     */
    static public function loginForm($action, $submitText = 'OK') {
        $texte_par_defaut = 'login' ;
        // Le formulaire
        return <<<HTML
<!--
Le formulaire est envoy selon la mthode GET  des fins de comprhension.
Il faut utiliser la mthode POST dans la pratique.
-->
<form name='auth' action='{$action}' method='POST' autocomplete='off'>
  <div>
    <input type='text' name='login' value='{$texte_par_defaut}'
           onClick="if (this.value == '$texte_par_defaut') this.value = ''"
           onFocus="if (this.value == '$texte_par_defaut') this.value = ''">
    <input type='password' name='pass'  >
    <input type='submit'   value='{$submitText}'>
  </div>
</form>
HTML;
    }

    /**
     * Validation de la connexion de l'utilisateur
     * @param array $data tableau contenant les cls 'login' et 'pass' associes au login et au mot de passe
     * @throws AuthenticationException si l'authentification choue
     *
     * @return User utilisateur authentifi
     */
    public static function createFromAuth(array $data) {
        if (!isset($data['login']) || !isset($data['pass'])) {
            throw new AuthenticationException("pas de login/pass fournis") ;
        }

        // Prparation de la requte
         $stmt = myPDO::getInstance()->prepare(<<<SQL
    SELECT id, firstName, lastName, login, phone, admin
    FROM tb_user
    WHERE login    = :login
    AND   sha1pass = SHA1(:pass)
SQL
    ) ;

        $stmt->execute(array(
            ':login' => $data['login'],
            ':pass'  => $data['pass'])) ;
        // Test de russite de la slection
        $stmt->setFetchMode(PDO::FETCH_CLASS, __CLASS__) ;
        if (($utilisateur = $stmt->fetch()) !== false) {
            self::startSession() ;
            $_SESSION[self::session_key]['connected'] = true ;
            return $utilisateur ;
        }
        else {
            throw new AuthenticationException("Login/pass incorrect") ;
        }
    }

    /**
     * Dconnecter l'utilisateur
     *
     * @return void
     */
    public static function logoutIfRequested() {
        if (isset($_REQUEST['logout'])) {
            self::startSession() ;
            unset($_SESSION[self::session_key]) ;
        }
    }

    /**
     * Un utilisateur est-il connect ?
     *
     * @return bool connect ou non
     */
    static public function isConnected() {
        self::startSession() ;
        return (isset($_SESSION[self::session_key]['connected']) && $_SESSION[self::session_key]['connected']) || (isset($_SESSION[self::session_key]['user']) && $_SESSION[self::session_key]['user'] instanceof User) ;
    }

    /**
     * Un utilisateur est-il admin ?
     *
     * @return bool
     */
    public function isAdmin() {
        $res = false;
        if($this->admin == 1) {
            $res=true;
        }
        return $res;
    }

    /**
     * Sauvegarde de l'objet Utilisateur dans la session
     *
     * @return void
     */
    public function saveIntoSession() {
        // Mise en place de la session
        self::startSession() ;
        // Mmorisation de l'Utilisateur
        $_SESSION[self::session_key]['user'] = $this ;
    }

    /**
     * Lecture de l'objet User dans la session
     * @throws NotInSessionException si l'objet n'est pas dans la session
     *
     * @return User
     */
    static public function createFromSession() {
        // Mise en place de la session
        self::startSession() ;
        // La variable de session existe ?
        if (isset($_SESSION[self::session_key]['user'])) {
            // Lecture de la variable de session
            $u = $_SESSION[self::session_key]['user'] ;
            // Est-ce un objet et un objet du bon type ?
            if (is_object($u) && get_class($u) == get_class()) {
                // OUI ! on le retourne
                return $u ;
            }
        }
        // NON ! exception NotInSessionException
        throw new NotInSessionException() ;
   }

   /**
    * Dmarrer une session
    * @throws SessionException si la session ne peut tre dmarre
    *
    * @return void
    */
    static protected function startSession($minutes = 0) {
        // Vision la plus contraignante et donc la plus fiable
        // Si les en-ttes ont deja t envoys, c'est trop tard...
		/*
        if (headers_sent())
            throw new SessionException("Impossible de dmarrer une session si les en-ttes HTTP ont t envoys") ;
        // Si la session n'est pas demarre, le faire
        if (!session_id()) session_start() ;
		*/
		
        // Vision la moins contraignante qui peut amener des comportements changeants
        // Si la session n'est pas demarre, le faire
        if (!session_id()) {
            // Si les en-ttes ont deja t envoys, c'est trop tard...
            if (headers_sent())
                throw new Exception("Impossible de dmarrer une session si les en-ttes HTTP ont t envoys") ;
            // Dmarrer la session
            session_start() ;
        }
        
   }

    /**
     * Production d'un code alatoire (minuscule, majuscule et chiffre)
     * @param $size taille de la chane
     *
     * @return string chane alatoire
     */
    public static function randomString($size) {
        $s = '' ;
        for ($i=0; $i<$size; $i++) {
            switch (rand(0, 2)) {
            case 0 :
                $s .= chr(rand(ord('A'), ord('Z'))) ;
                break ;
            case 1 :
                $s .= chr(rand(ord('a'), ord('z'))) ;
                break ;
            case 2 :
                $s .= chr(rand(ord('1'), ord('9'))) ;
                break ;
            }
        }
        return $s ;
    }

    /**
     * Production d'un formulaire de connexion contenant un challenge et une mthode JavaScript de hachage
     * @param string $action URL cible du formulaire
     * @param string $submitText texte du bouton d'envoi
     *
     * @return string code HTML du formulaire
     */
    static public function loginFormSHA1($action, $submitText = 'OK') {
        $texte_par_defaut = 'login' ;
        // Mise en place de la session
        self::startSession() ;
        // Mmorisation d'un challenge dans la session
        $_SESSION[self::session_key]['challenge'] = self::randomString(16) ;
        // Le formulaire avec le code JavaScript permettant le hachage SHA1
        // Le retour attendu par le serveur est SHA1(SHA1(pass)+challenge+SHA1(login))
        return <<<HTML
<script type='text/javascript' src='sha1.js'></script>
<script type='text/javascript'>
function crypter(f, challenge) {
    if (f.login.value.length && f.pass.value.length) {
        f.code.value = SHA1(SHA1(f.pass.value)+challenge+SHA1(f.login.value)) ;
        f.login.value = f.pass.value = '' ;
        return true ;
    }
    return false ;
}
</script>
<!--
Le formulaire est envoy selon la mthode GET  des fins de comprhension.
Il faut utiliser la mthode POST dans la pratique.
-->
<form name='auth' action='$action' method='POST' onSubmit="return crypter(this, '{$_SESSION[self::session_key]['challenge']}')" autocomplete='off'>
	<fieldset>
		<legend>Connection</legend>
		<input type='text' name='login' value='{$texte_par_defaut}'
		 onClick="if (this.value == '$texte_par_defaut') this.value = ''"
		 onFocus="if (this.value == '$texte_par_defaut') this.value = ''">
		<input type='password' name='pass'>
		<input type='hidden'   name='code'>
		<input type='submit'   value='{$submitText}'>
   </fieldset>
</form>
HTML;
    }

    /**
     * Validation de la connexion de l'Utilisateur
     * @param array $data tableau contenant la cl 'code' associe au condensat du login et au mot de passe
     * @throws AuthenticationException si l'authentification choue
     *
     * @return User utilisateur authentifi
     */
    public static function createFromAuthSHA1(array $data) {
        if (!isset($data['code'])) {
            throw new AuthenticationException("pas de login/pass fournis") ;
        }

        // Mise en place de la session
        self::startSession() ;
        // Prparation de la requte
        $stmt = myPDO::getInstance()->prepare(<<<SQL
    SELECT id, firstName, lastName, login, phone, admin
    FROM tb_user
    WHERE SHA1(CONCAT(sha1pass, :challenge, SHA1(login))) = :code
SQL
    ) ;

        $stmt->execute(array(
            ':challenge' => isset($_SESSION[self::session_key]['challenge']) ? $_SESSION[self::session_key]['challenge'] : '',
            ':code'      => $data['code'])) ;
        // Effacement du challenge
        // Toute la scurit repose sur lui, il doit rester valide le moins longtemps possible
        unset($_SESSION[self::session_key]['challenge']) ;
        // Test de russite de la slection
        $stmt->setFetchMode(PDO::FETCH_CLASS, __CLASS__) ;
        if (($utilisateur = $stmt->fetch()) !== false) {
            return $utilisateur ;
        }
        else {
            throw new AuthenticationException("Login/pass incorrect") ;
        }
    }

    /**
     * Retourner les pictogramme d'un utilisateur par ordre alphabetique
     * @return array<Picto> tableau d'instances de Picto
     */
    public function getPicts()
    {
      $idUser=$this->id;
      $dbh=myPDO::getInstance();
      $sth = $dbh->prepare(<<< SQL
SELECT pct_id, pct_nom, pct_img, pct_snd, pct_cat_id
FROM tb_picto
INNER JOIN tb_picto_utilisateur
ON pct_id = pcu_pct_id
WHERE pcu_utl_id={$idUser}
ORDER BY pct_nom
SQL
);
      $sth->setFetchMode(PDO::FETCH_CLASS,'Picto');
      $sth->execute();
      $res = $sth->fetchAll();
      return $res;
    }
}